@@ -9,7 +9,12 @@ use Scalar::Util qw(blessed);
 
 sub new {
     my ($class) = @_;
-    return bless {entry => undef, submap => {}}, $class;
+    
+    return bless {
+        entry    => undef,
+        submap   => {},
+        password => undef,
+    }, $class;
 }
 
 sub entry {
@@ -63,6 +68,13 @@ sub traverse {
     $visit->($self, 0);
 }
 
+sub password {
+    my $self = shift;
+    my $password = $self->{password};
+    $self->{password} = shift if @_;
+    return $password;
+}
+
 sub _descend_path {
     my ($self, $spec, $callback) = @_;
     
@@ -94,14 +106,14 @@ sub _descend_path {
 sub _make_subnode {
     my ($self, $rdn) = @_;
     # E.g. $rdn == {ou => 'Sales'}
-    my $canonical = canonical_dn([$rdn], casefold => 'lower');
+    my $canonical = lc canonical_dn([$rdn], casefold => 'none');
     return $self->{submap}{$canonical} ||= ref($self)->new;
 }
 
 sub _get_subnode {
     my ($self, $rdn) = @_;
     # E.g. $rdn == {ou => 'Sales'}
-    my $canonical = canonical_dn([$rdn], casefold => 'lower');
+    my $canonical = lc canonical_dn([$rdn], casefold => 'none');
     return $self->{submap}{$canonical};
 }
 
