@@ -6,13 +6,14 @@ package Test::Net::LDAP::Util;
 use base 'Exporter';
 use Net::LDAP;
 use Net::LDAP::Constant qw(LDAP_SUCCESS);
-use Net::LDAP::Util qw(ldap_error_name ldap_error_text);
+use Net::LDAP::Util qw(ldap_error_name ldap_error_text canonical_dn);
 use Test::Builder;
 
 our @EXPORT_OK = qw(
     ldap_result_ok
     ldap_result_is
     ldap_mockify
+    ldap_dn_is
 );
 
 our %EXPORT_TAGS = (all => \@EXPORT_OK);
@@ -31,6 +32,7 @@ The following subroutines are exported on demand.
         ldap_result_ok
         ldap_result_is
         ldap_mockify
+        ldap_dn_is
     );
 
 All the subroutines are exported if C<:all> is specified.
@@ -43,6 +45,15 @@ All the subroutines are exported if C<:all> is specified.
 
 =cut
 
+sub _format_diag {
+    my ($actual_text, $expected_text) = @_;
+
+    # Indent spaces are based on Test::Builder::_is_diag implementation
+    # ($Test::Builder::VERSION == 0.98)
+    return sprintf("%12s: %s\n", 'got', $actual_text).
+           sprintf("%12s: %s\n", 'expected', $expected_text);
+}
+
 =head2 ldap_result_ok
 
     ldap_result_ok($mesg, $name);
@@ -95,13 +106,8 @@ sub ldap_result_is {
             ((ref $actual && $actual->error) || ldap_error_text($actual));
         
         my $expected_text = ldap_error_name($expected).' ('.$expected_code.')';
-        
-        # Indent spaces are based on Test::Builder::_is_diag implementation
-        # ($Test::Builder::VERSION == 0.98)
-        $test_builder->diag(
-            sprintf("%12s: %s\n", 'got', $actual_text).
-            sprintf("%12s: %s\n", 'expected', $expected_text)
-        );
+
+        $test_builder->diag(_format_diag($actual_text, $expected_text));
     }
     
     return $actual;
@@ -126,4 +132,49 @@ sub ldap_mockify(&) {
     $callback->();
 }
 
+=head2 ldap_dn_is
+
+    ldap_dn_is($actual_dn, $expect_dn, $name);
+
+Tests equality of two DNs that are not necessarily canonicalized.
+
+The comparison is case-insensitive.
+
+=cut
+
+sub ldap_dn_is {
+    my ($actual_dn, $expected_dn, $name) = @_;
+    my ($actual_canonical_dn, $expected_canonical_dn) = ($actual_dn, $expected_dn);
+
+    for my $dn ($actual_canonical_dn, $expected_canonical_dn) {
+        $dn = lc canonical_dn($dn, casefold => 'none') if defined $dn;
+    }
+
+    my $success;
+
+    if (defined $actual_dn) {
+        if (defined $expected_dn) {
+            $success = $actual_canonical_dn eq $expected_canonical_dn;
+        } else {
+            $success = 0;
+        }
+    } else {
+        $success = !defined $expected_dn;
+    }
+
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    $test_builder ||= Test::Builder->new;
+    $test_builder->ok($success, $name);
+
+    unless ($success) {
+        my ($actual_text, $expected_text) = ($actual_dn, $expected_dn);
+
+        for my $text ($actual_text, $expected_text) {
+            $text = defined $text ? "'$text'" : 'undef';
+        }
+
+        $test_builder->diag(_format_diag($actual_text, $expected_text));
+    }
+}
+
 1;
